// back.c

#include "main.h"
#include "back_geom.h"

#define ATTRIBUTE_CLOSENESS 40000
#define VERSION "0.3"

int testDate = 1;
Bool doTest = False;

//window initialisation
Window	root, win[2];
GC	a_gc;

int activeWin;  //The window that will be drawn in (dockapp or tile)

char *geometry;
unsigned int borderwidth;

int screen;
int x_fd;

XSizeHints a_sizehints;
XWMHints   a_wmhints;
Pixel	   fore_pix, back_pix;

//pixmap stuff

time_t  actualtime;
int     weekday, day, month;

Pixmap back, disp, pix_mask;
#ifndef WMAKER
Pixmap tile;
#endif

XpmAttributes attributes;

char* progName;

//Local functions
void    repaint ();
void    drawAll ();
void    drawDate ();
void    calcDate ();
void	readArgs (int argc, char *argv[]);
void	ButtonPressEvent (XButtonEvent *x_ev);

/*************************************************************************/

int main (int argc, char *argv[])
{
  Bool finished;
  int old_day;

/*
 * Initialization of X-display
 */
  char *display_name = NULL;
  char *wname = NAME;

  XGCValues gcv;
  unsigned long gcm;
  XEvent event;
  XTextProperty name;
  XClassHint classhint;

  readArgs(argc, argv);

/*
 * Try to open display
 */
  if ( !( dpy = XOpenDisplay (display_name)))
  {
    fprintf (stderr, "Can't open display !!!");
    exit (1);
  }
  screen = DefaultScreen (dpy);
  root = RootWindow (dpy, screen);
  x_fd = XConnectionNumber (dpy);

/*
 * Create a GC
 */
  gcm = GCForeground | GCBackground | GCGraphicsExposures;
  gcv.graphics_exposures = False;
  a_gc = XCreateGC (dpy, root, gcm, &gcv);


/*
 * Convert xpm's to usable ximages
 */
  attributes.valuemask = XpmExactColors|XpmCloseness;
  attributes.exactColors = False;
  attributes.closeness = ATTRIBUTE_CLOSENESS;
  XpmCreatePixmapFromData (dpy, root, back_xpm, &back, &pix_mask, &attributes);
#ifndef WMAKER
  XpmCreatePixmapFromData (dpy, root, tile_xpm, &tile, NULL, &attributes);
#endif
  disp = XCreatePixmap(dpy, root, TWIDTH, THEIGHT, DefaultDepth(dpy,DefaultScreen(dpy)));

  fore_pix = GetColor ("black");
  back_pix = GetColor ("white");


#ifdef WMAKER
  a_sizehints.flags = PPosition | PMinSize | PMaxSize;
#else
  a_sizehints.flags = PMinSize | PMaxSize;
#endif
  a_sizehints.x = TLEFT;
  a_sizehints.y = TTOP;
  a_sizehints.min_width = TWIDTH;
  a_sizehints.min_height = THEIGHT;
  a_sizehints.max_width = TWIDTH;
  a_sizehints.max_height = THEIGHT;

  win[0] = XCreateSimpleWindow (dpy, root, a_sizehints.x, a_sizehints.y, TWIDTH, THEIGHT, 0, fore_pix, back_pix);
  win[1] = XCreateSimpleWindow (dpy, root, a_sizehints.x, a_sizehints.y, TWIDTH, THEIGHT, 0, fore_pix, back_pix);

  XSetWMNormalHints (dpy, win[0], &a_sizehints);

  a_wmhints.window_group = win[0];
#ifdef WMAKER
  a_wmhints.initial_state = WithdrawnState;
  a_wmhints.icon_window = win[1];
  a_wmhints.flags = StateHint|IconWindowHint|WindowGroupHint;
  activeWin = 1;
#else
  a_wmhints.initial_state = NormalState;
  a_wmhints.flags = StateHint|WindowGroupHint;
  activeWin = 0;
#endif
  XSetWMHints(dpy, win[0], &a_wmhints);

  classhint.res_name = NAME;
  classhint.res_class = CLASS;
  XSetClassHint (dpy, win[0], &classhint);

  if (XStringListToTextProperty(&wname, 1, &name) ==0) {
    fprintf(stderr, "Can't allocate window name\n");
    exit(-1);
  }
  XSetWMName(dpy, win[0], &name);

#ifdef WMAKER
  XShapeCombineMask (dpy, win[activeWin], ShapeBounding, TLEFT, TTOP, pix_mask, ShapeSet);
#else
  XCopyArea (dpy, tile, disp, a_gc, TLEFT, TTOP, TWIDTH, THEIGHT, TLEFT, TTOP);
#endif

  XSetClipMask (dpy, a_gc, pix_mask);
  XCopyArea (dpy, back, disp, a_gc, TLEFT, TTOP, TWIDTH, THEIGHT, TLEFT, TTOP);
  XSetClipMask (dpy, a_gc, None);

  XSetCommand (dpy, win[0], argv, argc);
  XStoreName (dpy, win[0], NAME);
  XSetIconName (dpy, win[0], NAME);


  XMapWindow (dpy, win[0]);


/*
 * ---------------------------------------
 *|    M A I N   L O O P - Starts here    |
 * ---------------------------------------------------------------------------
 */

  calcDate ();
  drawAll ();

  XSelectInput (dpy, win[activeWin], ExposureMask | ButtonPressMask);

  old_day = 0;  //So old_day and day will initially be different

  finished = False;
  while (!finished)
  {
    while (XPending (dpy))
    {
      XNextEvent (dpy, &event);
      switch (event.type)
      {
        case Expose:            //If window has been "not visible"
          repaint();
          break;
        case DestroyNotify:     //When X wants it to close   -> does this "work" ?
          fprintf (stderr, "Destroy notify event...\n");
          XCloseDisplay (dpy);
          exit(0);
	case ButtonPress:	//A mouse-button was pressed
	  if (doTest) ButtonPressEvent (&event.xbutton);
	  break;
        default:
          break;
      }
    }
    XFlush (dpy);

#ifdef SYSV
      poll ( (struct poll *) 0, (size_t) 0, 50);
#else
      usleep (50000L);         //sleep half a second
#endif
      calcDate (weekday, day, month);
      if (day != old_day)
      {
        drawDate (weekday, day, month);
        repaint ();
        old_day = day;
      }
  }
fprintf (stderr, "Closing...");
  XFreeGC (dpy, a_gc);
  XFreePixmap (dpy, disp);
  XFreePixmap (dpy, pix_mask);
#ifndef WMAKER
  XFreePixmap (dpy, tile);
#endif
  XDestroyWindow (dpy, win[1]);
  XDestroyWindow (dpy, win[0]);
  XCloseDisplay (dpy);

  return 0;
}



Pixel GetColor(char *name)
{
  XColor color;
  XWindowAttributes attributes;

  XGetWindowAttributes(dpy, root, &attributes);
  color.pixel = 0;
   if (!XParseColor (dpy, attributes.colormap, name, &color))
     {
       fprintf (stderr, "Can not parse %s", name);
     }
   else if(!XAllocColor (dpy, attributes.colormap, &color))
     {
       fprintf (stderr, "Can not allocate %s",name);
     }
  return color.pixel;
}



void repaint()
{
   XEvent xev;
   XCopyArea (dpy, disp, win[activeWin], a_gc, TLEFT, TTOP, TWIDTH, THEIGHT, TLEFT, TTOP);
   while(XCheckTypedEvent(dpy, Expose, &xev));
}


void drawAll ()
{
  XCopyArea (dpy, back, disp, a_gc, TLEFT, TTOP, TWIDTH, THEIGHT, TTOP, TLEFT);
  drawDate (weekday, day, month);
}


void drawDate ()
{
  int day10, day1;
  int offset10, offset1;
  div_t dummy;
  int width, width10, width1;

  if (!doTest)
     dummy = div (day, 10);
  else
     dummy = div (testDate, 10);

  day10 = dummy.quot;
  day1  = dummy.rem;

  width10 = D_WID;
  width1  = D_WID;  

  if (day10 == 0) width10 = 0;
  if (day10 == 1) width10 = D_ONE;
  if (day1 == 1)  width1  = D_ONE;
  width = (width1 + width10) / 2;
  offset10 = 32 - width;
  offset1  = offset10 + width10;

  //Clear the area
  XCopyArea (dpy, back, disp, a_gc, 0,0,64,64,0,0);

  //Write day of week
  XCopyArea (dpy, back, disp, a_gc, TW_LFT, W_HEI * weekday, W_WID, W_HEI, PW_LFT, PW_TOP);

  //Write day of month
  if (day > 9)
  {
    XCopyArea (dpy, back, disp, a_gc, D_WID * day10 + (D_WID - width10), TD_TOP, 
                                      width10, D_HEI, 
                                      offset10, PD_TOP);
    XCopyArea (dpy, back, disp, a_gc, D_WID * day1 + (D_WID - width1), TD_TOP, 
                                      width1, D_HEI, 
                                      offset1, PD_TOP);
  }
  else
    XCopyArea (dpy, back, disp, a_gc, D_WID *day + (D_WID - width1), TD_TOP, 
	       width1, D_HEI, 32-width, PD_TOP);

  //Write month
  XCopyArea (dpy, back, disp, a_gc, TM_LFT, M_HEI*month, M_WID, M_HEI, PM_LFT, PM_TOP);
}


void calcDate ()
{
  static struct tm *clk;

  actualtime = time(0);
  clk = localtime (&actualtime);

  weekday = clk->tm_wday;
  day     = clk->tm_mday;
  month   = clk->tm_mon;
}


void ButtonPressEvent (XButtonEvent *x_ev)
{
   testDate++;
   if (testDate > 31) testDate = 1;

   drawDate();
   repaint ();
}


void readArgs (int argc, char *argv[])
{
   int i;

   for (i=1; i<argc;i++)
   {
      if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i],"-help")==0 || 
	  strcmp(argv[i],"--help")==0)
      {
	 fprintf(stderr, "\nwmDate - version 0.3.\n");
	 fprintf(stderr, "A utility to show today's date.\n\n");
	 fprintf(stderr, "Usage: %s [options]\n", argv[0]);
	 fprintf(stderr, "   -h | -help | --help   display this help screen.\n");
	 fprintf(stderr, "   -t | --test           go to test mode.\n");
	 fprintf(stderr, "   -v | --version        display version.\n\n");
	 exit(0);
      }
      if (strcmp(argv[i], "-t") == 0 || strcmp(argv[i], "--test") == 0) doTest = True;
      if (strcmp(argv[i], "-v") == 0 || strcmp(argv[i], "--version") == 0) 
      {
	 fprintf(stderr, "\n%s version %s\n\n", argv[0], VERSION);
	 exit(0);
      }

   }
}
